(* -*- tuareg -*-
 * OBus_config.ml
 * --------------
 * Copyright : (c) 2009, Jeremie Dimino <jeremie@dimino.org>
 * Licence   : BSD3
 *
 * This file is a part of obus, an ocaml implementation of D-Bus.
 *)

(* Localtion of the machine id file: *)
let machine_uuid_file = "/var/lib/dbus/machine-id"

(* Version of obus: *)
let version = "1.2.0"
